
package account;

import static org.valid4j.Assertive.*;
import org.valid4j.errors.*;

/**
 * Implements the IAccount interface Uses Valid4J to implement the pre- and
 * post-conditions
 * 
 * @author Horst Lichter, Jochen Ludewig Software Engineering  Grundlagen,
 *         Menschen, Prozesse, Techniken, dpunkt.verlag (c) 2023
 */
public class AccountImplValid4J implements IAccount {

   private int overdraft = 0;
   private int balance = 0;
   private String holder = "";

   /**
    * @param amount initial deposited amount
    * @param owner  of the account
    * @param overdr amount of the overdraft limit
    * @throws ContractViolation
    */
   public AccountImplValid4J(int amount, String owner, int overdr) throws ContractViolation {
      require(amount >= 0, "pre-v: amount (%o) must be positive", amount);
      require(overdr >= 0, "pre-v: overdraft (%o) must be negative", overdr);
      require(!owner.isEmpty(), "pre-v: new holder must not be empty");
      balance = amount;
      holder = owner;
      overdraft = overdr;
      ensure(balance == amount);
      ensure(holder == owner);
      ensure(overdraft == overdr);
      ensure(invariant(), "inv-v");
   }

   public int balance() {
      require(invariant(), "inv-v");
      return balance;

   }

   public void deposit(int amount) throws ContractViolation {
      require(invariant(), "inv-v");
      require(amount > 0, "pre-v: amount (%o) must be positive", amount);

      int balancePRE = balance;
      balance = balance + amount;

      ensure(balance == balancePRE + amount, "post-v: wrong balance computation");
      ensure(invariant(), "inv-v");
   }

   public void withdraw(int amount) throws ContractViolation {
      require(invariant(), "inv-v");
      require(amount > 0, "pre-v: amount (%o) must be positive", amount);
      require(balance - amount > overdraft, "pre-v: amount (%o) too high", amount);

      int balancePRE = balance;
      balance = balance - amount;

      ensure(balance == balancePRE - amount, "post-v: wrong balance computation");
      ensure(invariant(), "inv-v");
   }

   public String getHolder() throws ContractViolation {
      require(invariant(), "inv-v");
      return holder;
   }

   public void setHolder(String newHolder) throws ContractViolation {
      require(invariant(), "inv-v");
      require(!newHolder.isEmpty(), "pre-v: new holder must not be empty");
      require(!holder.equals(newHolder), "pre-v: new holder (%s) must must have a different name", newHolder);

      holder = newHolder;

      ensure(holder.equals(newHolder), "post-v: holder (%s) incorrect", holder);
      ensure(invariant(), "inv-v");

   }

   public int overdraft() throws ContractViolation {
      require(invariant(), "inv-v");
      return overdraft;
   }

   protected boolean invariant() {
      return (balance >= overdraft && !holder.isEmpty());
   }

}
